import type { Context } from 'hono';
import type { AuthorizationRequest, OIDCProviderConfig } from '../types';
import { PasswordAuthStrategy, type AuthenticationResult, type PasswordAuthConfig } from './strategies/password-auth-strategy';

/**
 * 密码验证器类型
 */
export type PasswordValidator = (username: string, password: string) => Promise<string | null>;

/**
 * 认证管理器配置
 */
export interface AuthManagerConfig {
  /** 会话TTL（秒） */
  sessionTTL?: number;
  /** 记住我最大存活时间（秒） */
  rememberMeMaxAge?: number;
  /** 页面配置 */
  pageConfig?: {
    title?: string;
    brandName?: string;
    logoUrl?: string;
  };
}

/**
 * 认证管理器
 * 专门处理密码认证
 */
export class AuthManager {
  private readonly passwordAuth: PasswordAuthStrategy;

  constructor(
    oidcConfig: OIDCProviderConfig,
    passwordValidator: PasswordValidator,
    config: AuthManagerConfig = {}
  ) {
    const passwordConfig: PasswordAuthConfig = {
      sessionTTL: config.sessionTTL || 24 * 60 * 60, // 默认24小时
      rememberMeMaxAge: config.rememberMeMaxAge || 30 * 24 * 60 * 60, // 默认30天
      pageConfig: config.pageConfig || {}
    };

    this.passwordAuth = new PasswordAuthStrategy(
      oidcConfig,
      passwordValidator,
      passwordConfig
    );
  }

  /**
   * 检查当前用户是否已认证
   */
  async getCurrentUser(c: Context): Promise<string | null> {
    return await this.passwordAuth.getCurrentUser(c);
  }

  /**
   * 处理认证要求
   */
  async handleAuthenticationRequired(
    c: Context,
    authRequest: AuthorizationRequest
  ): Promise<Response> {
    return await this.passwordAuth.handleAuthenticationRequired(c, authRequest);
  }

  /**
   * 处理认证请求
   */
  async authenticate(c: Context): Promise<AuthenticationResult> {
    return await this.passwordAuth.authenticate(c);
  }

  /**
   * 处理登出请求
   */
  async logout(c: Context): Promise<Response> {
    return await this.passwordAuth.logout(c);
  }

  /**
   * 处理完整的登录流程
   */
  async handleLogin(c: Context, authRequest: AuthorizationRequest): Promise<Response> {
    try {
      // 执行认证
      const authResult = await this.passwordAuth.authenticate(c);

      if (authResult.success) {
        // 认证成功，处理后续操作
        return await this.handleAuthenticationSuccess(c, authResult);
      } else {
        // 认证失败，返回错误页面
        return await this.handleAuthenticationFailure(c, authResult, authRequest);
      }
    } catch (error) {
      console.error('登录流程处理失败:', error);

      // 返回通用错误页面
      return await this.handleAuthenticationFailure(
        c,
        { success: false, error: '服务器内部错误' },
        authRequest
      );
    }
  }

  /**
   * 检查认证状态并处理
   */
  async checkAuthenticationStatus(
    c: Context,
    authRequest: AuthorizationRequest
  ): Promise<{ authenticated: boolean; userId?: string; response?: Response }> {
    try {
      const userId = await this.getCurrentUser(c);

      if (userId) {
        return {
          authenticated: true,
          userId
        };
      } else {
        const response = await this.handleAuthenticationRequired(c, authRequest);
        return {
          authenticated: false,
          response
        };
      }
    } catch (error) {
      console.error('检查认证状态失败:', error);
      throw error;
    }
  }

  /**
   * 处理认证成功
   */
  private async handleAuthenticationSuccess(
    c: Context,
    result: AuthenticationResult
  ): Promise<Response> {
    return await this.passwordAuth.handleAuthenticationSuccess(c, result);
  }

  /**
   * 处理认证失败
   */
  private async handleAuthenticationFailure(
    c: Context,
    result: AuthenticationResult,
    authRequest: AuthorizationRequest
  ): Promise<Response> {
    return await this.passwordAuth.handleAuthenticationFailure(c, result, authRequest);
  }
} 